      program marcstoatlas
      
C
C  Program to read the MARCS model atmosphere (calling the subroutine 'readmodel') 
C  and to write it in the standard ATLAS9 format     
C 
C  This is a modified version (A.Mucciarelli, August 2012) of the original code 
C  by Bengt Edvardsson (Version 1.2 3-Apr-2012)     
C
C  Run by  "cat model_file | marcstoatlas"  ---> the output is written in fort.88
C

      implicit none
      integer ndp,nmolmax,k
      parameter(ndp=100)
      parameter (nmolmax=30)
      character fileID*72
***
      real*4 teff,flux,g,xite,rmass,rmetal,alpha,radius,rluminosity
      real*4 palfa,pny,py,pbeta,abund(92),xx,yy,zz
      common /global/ teff,flux,g,xite,rmass,rmetal,alpha,radius,
     &                rluminosity,palfa,pny,py,pbeta,abund,xx,yy,zz
***
      integer ndepth
      real*4 tau(ndp),taus(ndp),z(ndp),t(ndp),pe(ndp),pg(ndp),prad(ndp)
      real*4 pturb(ndp),xkapr(ndp),ro(ndp),emu(ndp),v(ndp),frconv(ndp)
      real*4 rhox(ndp)
      real*4 accrad(ndp),xne(ndp)
      common /structure/ ndepth,tau,taus,z,t,pe,pg,prad,pturb,xkapr,
     &                   ro,emu,v,frconv,rhox
***
      real*4 hnic(ndp),other(ndp),presmo(nmolmax,ndp)
      common /pressures/ hnic,presmo,other
***
      real*4 xfraction, yfraction
      real*4 abum2(6)
      integer codex(6),j

      call readmodel(fileID)

       yfraction=yy/(4-3*yy)
       xfraction=1-yfraction
       g=(log10(g))
       

C------ First ATLAS9 block: main information
       
       write(88,'(A4,2x,F6.0,2x,A7,1x,F7.5,1x,A3)')
     &'TEFF',teff,'GRAVITY',g,'LTE'

       write(88,'(A18)')'TITLE MARCStoATLAS'
       write(88,'(A53)')
     &' OPACITY IFOP 1 1 1 1 1 1 1 1 1 1 1 1 1 0 1 0 0 0 0 0'
       write(88,'(A59)')
     & 'CONVECTION ON   1.50 TURBULENCE OFF  0.00  0.00  0.00  0.00'


C------ Second ATLAS9 block: abundances

       write(88,'(A15,F10.5,A20,F7.5,A3,F7.5)')
     &'ABUNDANCE SCALE',10**rmetal,' ABUNDANCE CHANGE 1 ',xfraction,
     &' 2 ',yfraction

       
       do J=2,ndepth-1       
        ACCRAD(J)=0.5*( (prad(J)-prad(J-1) ) / (RHOX(J)-
     &  RHOX(J-1))+
     & (prad(J+1)-prad(J) )/(RHOX(J+1)-RHOX(J) ) )
     
        ACCRAD(1)=(prad(J)-prad(J-1))/(RHOX(J)-
     &  RHOX(J-1))
        ACCRAD(ndepth)=(prad(J)-prad(J-1))/(RHOX(J)-
     &  RHOX(J-1))            
       enddo
        
       
       do k=3,93,6
         do j=1,6
	   abum2(j)=abund(k-1+j)-rmetal-12.04
	   codex(j)=(k-1)+j
	   if(abum2(j).lt.-20)abum2(j)=-20.00

	 enddo
	 write(88,'(A17,6(I3,F7.2))')
     &     'ABUNDANCE CHANGE',codex(1),abum2(1),
     &     codex(2),abum2(2),codex(3),abum2(3),
     &     codex(4),abum2(4),codex(5),abum2(5),
     &     codex(6),abum2(6)
       enddo
       
       write(88,'(A27)')'ABUNDANCE CHANGE  99 -20.00'


C------ Third ATLAS9 block: model atmosphere
       
       write(88,'(A10,I3,A33)')'READ DECK6 ',ndepth,
     &'RHOX,T,P,XNE,ABROSS,ACCRAD,VTURB'
       
       do k=1,ndepth       
        write(88,'(e15.8,2x,f7.1,5(1x,e9.3))')
     &       rhox(k),t(k),pg(k),
     &      (Pe(k))/(1.38054e-16*t(k)),(xkapr(k)),
     &       ACCRAD(k),xite*1e5
       enddo
       
       write(88,'(A7,E10.4)')'PRADK0 ',PRAD(1)
       write(88,'(A48)')
     &'BEGIN                    ITERATION  15 COMPLETED'



      end
      
 
 
      
      
      subroutine readmodel(fileID)
* reads a MARCS model and stores data into arrays
* in 3 common blocks: 'global', 'structure' and 'pressures'
*
**********************************************************************
* fileID returns the model ID string (a72)
**********************************************************************
* Global properties of the model: common "global"
* teff        = effective temperature [K]
* flux        = physical flux [erg/cm2/s]
* g           = surface acceleration of gravity at tau(Rosseland)=1.0 [cm/s2]
* xite        = microturbulence parameter [km/s]
* rmass       = stellar mass [Msun] (0.0 for plane-parallel models)
* rmetal      = [Fe/H]
* alpha       = [alpha/Fe]
* radius      = Stellar radius [cm] at Tau(Rosseland) = 1.0
*               (1.0 cm for plane-parallel models)
* rluminosity = luminosity [Lsun] for the radius given above
* palfa pny py pbeta = convection parameters
*        Ref: Henyey L., Vardaya M.S., Bodenheimer P. 1965, ApJ 142, 841
* abund()     = logarithmic number abundances of the 92 first chemical
*               elements in the periodic table
* XX          = Hydrogen mass fraction
* YY          = Helium mass fraction
* ZZ          = Metal mass fraction
**************
* Model structure and thermodynamics, depth index = k: common "structure"
* ndepth      = number of depth points
* tau(k)      = log(tau(Rosseland))
* taus(k)     = log(tau(5000 A))
* z(k)        = depth [cm], depth=0.0 @ tau(Rosseland)=1.0
* t(k)        = temperature [K]
* pe(k)       = electron pressure [dyn/cm2]
* pg(k)       = gas pressure [dyn/cm2]
* prad(k)     = radiation pressure [dyn/cm2]
* pturb(k)    = turbulence pressure [dyn/cm2]
* xkapr(k)    = Rosseland opacity [cm2/g]
* ro(k)       = density [g/cm3]
* emu(k)      = mean molecular weight [amu]
* v(k)        = convective velocity [cm/s]
* frconv(k)   = Fractional convective flux Fconv/F
* rhox(k)     = Column mass above point k [g/cm2]
**************
* Partial pressures: common "pressures"
* hnic(k)     = log(H I pressure/[1 dyn/cm2])
* presmo(j,k) = log(XXX pressure/[1 dyn/cm2]) for j:XXX
*               1:H-, 2:H2, 3:H2+, 4:H2O, 5:OH, 6:CH, 7:CO,
*               8:CN, 9:C2, 10:N2, 11:O2, 12:NO, 13:NH,
*               14:TiO, 15:C2H2, 16:HCN, 17:C2H, 18:HS,
*               19:SiH, 20:C3H, 21:C3, 22:CS, 23:SiC,
*               24:SiC2, 25:NS, 26:SiN, 27:SiO, 28:SO, 29:S2, 30:SiS
* other(k)    = log(pressure of not listed gas constituents/[dyn/cm2])
**********************************************************************
      implicit none
      integer ndp,nmolmax
      parameter(ndp=100)
      parameter (nmolmax=30)
      character fileID*72
      real*4 taudum,pgas
      integer k,kk,j
***
      real*4 teff,flux,g,xite,rmass,rmetal,alpha,radius,rluminosity
      real*4 palfa,pny,py,pbeta,abund(92),xx,yy,zz
      common /global/ teff,flux,g,xite,rmass,rmetal,alpha,radius,
     &                rluminosity,palfa,pny,py,pbeta,abund,xx,yy,zz
***
      integer ndepth
      real*4 tau(ndp),taus(ndp),z(ndp),t(ndp),pe(ndp),pg(ndp),prad(ndp)
      real*4 pturb(ndp),xkapr(ndp),ro(ndp),emu(ndp),v(ndp),frconv(ndp)
      real*4 rhox(ndp)
      common /structure/ ndepth,tau,taus,z,t,pe,pg,prad,pturb,xkapr,
     &                   ro,emu,v,frconv,rhox
***
      real*4 hnic(ndp),other(ndp),presmo(nmolmax,ndp)
      common /pressures/ hnic,presmo,other
***
      read(*,'(a)') fileID
      read(*,1170) TEFF,FLUX,G,xite,rmass,rmetal,alpha,radius,
     &              rluminosity
     
     
 1170 format(f7.0,/,e12.4,/,e12.4,/,f5.1,/,f5.1,/,2f6.2,/,e12.4,/,e12.4)
      read(*,1176) PALFA,PNY,PY,PBETA
 1176 format(f6.2,f5.2,f6.3,f5.2)
      read(*,*) xx,yy,zz
 1178 format(x,3f8.5)
      read(*,*)
      read(*,1180) abund
 1180 format(10f7.2)
 
 
*
* structure
*
      read(*,2105) ndepth
 2105 format(i4)
      read(*,*)
      read(*,*)
      do kk=1,ndepth
        read(*,*) k,tau(k),taus(k),z(k),t(k),
     &               pe(k),pg(k),prad(k),pturb(k)
        if(kk.ne.k) stop 'model and loop indices differ'
      enddo
*
* thermodynamics
*
      read(*,*)
      do kk=1,ndepth
        read(*,*) k,taudum,xkapr(k),ro(k),emu(k),v(k),frconv(k),
     &               rhox(k)
        if(kk.ne.k) stop 'model and loop indices differ'
      enddo
*
* molecular partial pressures
*
      read(*,*)
      read(*,*)
      do kk=1,ndepth
        read(*,*) k,pgas,hnic(k),(presmo(j,k),j=1,9)
        if(kk.ne.k) stop 'model and loop indices differ'
      enddo
      read(*,*)
      do kk=1,ndepth
        read(*,*) k,(presmo(j,k),j=10,20)
        if(kk.ne.k) stop 'model and loop indices differ'
      enddo
      read(*,*)
      do kk=1,ndepth
        read(*,*) k,(presmo(j,k),j=21,30),other(k)
        if(kk.ne.k) stop 'model and loop indices differ'
      enddo

      return
      END
